/*
 * Async Treeview 0.1 - Lazy-loading extension for Treeview
 *
 * http://bassistance.de/jquery-plugins/jquery-plugin-treeview/
 *
 * Copyright (c) 2007 Jörn Zaefferer
 *
 * Dual licensed under the MIT and GPL licenses:
 *   http://www.opensource.org/licenses/mit-license.php
 *   http://www.gnu.org/licenses/gpl.html
 *
 * Revision: $Id$
 * Original by: XXX
 * Modified by: Nuxeo
 */

;(function($) {

function load(settings, root, child, container) {
  $.getJSON(settings.url, {root: root}, function(response) {
    function createNode(parent) {
        if (this.href) {
            var item = $("<a/>").attr('href', this.href).html(this.text)
        } else {
            var item = "<span>" + this.text + "</span>"
        }

      var current = $("<li/>").attr("id", this.id || "").html(item).appendTo(parent);
      if (this.classes) {
        current.children("span").addClass(this.classes);
      }
      if (this.expanded) {
        current.addClass("open");
      }
      if (this.hasChildren || this.children && this.children.length) {
        var branch = $("<ul/>").appendTo(current);
        branch.addClass('branch');
        if (this.hasChildren) {
          current.addClass("hasChildren");
          createNode.call({
            text:"(loading)",
            id:"placeholder",
            children:[]
          }, branch);
        }
        if (this.children && this.children.length) {
          $.each(this.children, createNode, [branch])
        }
      }
    }
    $.each(response, createNode, [child]);
    $(container).treeview({add: child});

    if (currentSelectedTreeId!='') {
        items = $("li").find(".open").filter(function() { return this.id==currentSelectedTreeId;})
        $($(items[0]).find("a")[0]).css("background-color","#DDDDEE").css("color","black").css("text-decoration","none").css("padding","5px");
    }

    });
}

var proxied = $.fn.treeview;
$.fn.treeview = function(settings) {
  if (!settings.url) {
    return proxied.apply(this, arguments);
  }
  var container = this;
  if (anonymousTree && currentSelectedTreeId!='') {
    load(settings, "source:" + currentSelectedTreeId  , this, container);
  }
  else {
    load(settings, "source", this, container);
  }
  var userToggle = settings.toggle;
  return proxied.call(this, $.extend({}, settings, {
    collapsed: true,
    toggle: function() {
      var $this = $(this);
      if ($this.hasClass("hasChildren")) {
        var childList = $this.removeClass("hasChildren").find("ul");
        childList.empty();
        load(settings, this.id, childList, container);
      }
      if (userToggle) {
        userToggle.apply(this, arguments);
      }
    }
  }));
};

})(jQuery);