package org.nuxeo.ecm.platform.sync.api.util;

import java.io.Serializable;

import org.nuxeo.ecm.platform.web.common.vh.VirtualHostHelper;

/**
 * Utility class that will keep all the details needed in the process of
 * synchronization
 *
 * @author rux
 *
 */
public class SynchronizeDetails implements Serializable {

    private static final long serialVersionUID = -3876136428566855181L;

    private String username;

    private String password;

    private String host;

    private int port = 8080;

    private String protocol = "http";

    private String contextPath;

    public SynchronizeDetails() {}

    public SynchronizeDetails(String username, String password, String protocol, String host,
            int port, String contextPath) {
        this.username = username;
        this.password = password;
        this.protocol = protocol;
        this.host = host;
        this.port = port;
        this.contextPath = contextPath;
    }

    public String getUrl() {
        StringBuilder url = new StringBuilder(getProtocol());
        url.append("://");
        url.append(getHost());
        url.append(":");
        url.append(getPort());
        url.append(getContextPath());
        return url.toString();
    }

    public String getUsername() {
        return username;
    }

    public void setUsername(String username) {
        this.username = username;
    }

    public String getPassword() {
        return password;
    }

    public void setPassword(String password) {
        this.password = password;
    }

    public String getHost() {
        return host;
    }

    public void setHost(String host) {
        this.host = host;
    }

    public int getPort() {
        return port;
    }

    public void setPort(int port) {
        this.port = port;
    }

    // XXX : is this used ?
    public void reset() {
        host = null;
        port = 8080;
        username = null;
        password = null;
    }

    public String getProtocol() {
        return protocol;
    }

    public void setProtocol(String protocol) {
        this.protocol = protocol;
    }

    public String getContextPath() {
        if (contextPath == null || "".equals(contextPath)) {
            contextPath = VirtualHostHelper.getContextPathProperty();
        }
        if (!contextPath.startsWith("/")) {
            contextPath = "/" + contextPath;
        }
        return contextPath;
    }

    public void setContextPath(String contextPath) {
        this.contextPath = contextPath;
    }

}
